#ifndef GMCXXSDK_STRATEGY_H
#define GMCXXSDK_STRATEGY_H

#include "gmapi.h"

#define GM_CLASS GM_CXX_API 

class GM_CLASS Strategy
{
public:
	Strategy(const char *token, const char *strategy_id, int mode);
	Strategy();
	virtual ~Strategy();

public: //

	//в
	int run();

	//ֹͣ
	void stop();

	//òID
	void set_strategy_id(const char *strategy_id);

	//ûtoken
	void set_token(const char *token);

	//òģʽ
	void set_mode(int mode);

	//ʱ
	int schedule(const char *data_rule, const char *time_rule);

	//ǰʱ
	double now();

	//ûز
	int set_backtest_config(
		const char  *start_time,
		const char  *end_time,
		double initial_cash = 1000000,
		double transaction_ratio = 1,
		double commission_ratio = 0,
		double slippage_ratio = 0,
		int    adjust = 0,
		int    check_cache = 1
	);

public: //ݺ

		// 
	int subscribe(const char *symbols, const char * frequency, bool unsubscribe_previous = false);

	// ˶
	int unsubscribe(const char *symbols, const char * frequency);


public: //׺

	//ѯ˺
	DataArray<Account>* get_accounts();

	//ѯָ˺״̬
	int get_account_status(const char *account, AccountStatus &as);

	//ѯн˺״̬
	DataArray<AccountStatus>* get_all_account_status();

	//ָί
	Order order_volume(const char *symbol, int volume, int side, int order_type, int position_effect, double price = 0, const char *account = NULL);

	//ֵָί
	Order order_value(const char *symbol, double value, int side, int order_type, int position_effect, double price = 0, const char *account = NULL);

	//ʲָί
	Order order_percent(const char *symbol, double percent, int side, int order_type, int position_effect, double price = 0, const char *account = NULL);

	//ֵĿֲ
	Order order_target_volume(const char *symbol, int volume, int position_side, int order_type, double price = 0, const char *account = NULL);

	//ֵĿֲֶ
	Order order_target_value(const char *symbol, double value, int position_side, int order_type, double price = 0, const char *account = NULL);

	//ֵĿֱֲʲı
	Order order_target_percent(const char *symbol, double percent, int position_side, int order_type, double price = 0, const char *account = NULL);

	//ƽǰпƽֲ
	DataArray<Order>* order_close_all();

	//ίг
	int order_cancel(const char *cl_ord_id, const char *account = NULL);

	//ί
	int order_cancel_all();

	//ίµ
	Order place_order(const char *symbol, int volume, int side, int order_type, int position_effect, double price = 0, int order_duration = 0, int order_qualifier = 0, double stop_price = 0, const char *account = NULL);

	//ѯί
	DataArray<Order>* get_orders(const char *account = NULL);

	//ѯδί
	DataArray<Order>* get_unfinished_orders(const char *account = NULL);

	//ѯɽ
	DataArray<ExecRpt>* get_execution_reports(const char *account = NULL);

	//ѯʽ
	DataArray<Cash>* get_cash(const char *accounts = NULL);

	//ѯֲ
	DataArray<Position>* get_position(const char *account = NULL);

	//ί㷨
	AlgoOrder order_algo(const char *symbol, int volume, int position_effect, int side, int order_type, double price, AlgoParam &algo_param, const char *account = NULL);

	//㷨ί
	int algo_order_cancel(const char *cl_ord_id, const char *account = NULL);

	//ͣ/ָ㷨
	int algo_order_pause(const char *cl_ord_id, int status, const char *account = NULL);

	//ѯ㷨ί
	DataArray<AlgoOrder>* get_algo_orders(const char *account = NULL);

	//ѯ㷨ί
	DataArray<Order>* get_algo_child_orders(const char *cl_ord_id, const char *account = NULL);


public: //Բຯ

	//Ӳ
	int add_parameters(Parameter *params, int count);

	//ɾ
	int del_parameters(const char *keys);

	//ò
	int set_parameters(Parameter *params, int count);

	//ȡ
	DataArray<Parameter>* get_parameters();

	//ñ
	int set_symbols(const char *symbols);

	//ȡ
	DataArray<Symbol>* get_symbols();


public: //¼

		//ʼ
		virtual void on_init();
		//յTick
		virtual void on_tick(Tick *tick);
		//յbar
		virtual void on_bar(Bar *bar);
		//յʳɽL2ʱЧ
		virtual void on_l2transaction(L2Transaction *l2transaction);
		//ίб仯
		virtual void on_order_status(Order *order);
		//ִлر
		virtual void on_execution_report(ExecRpt *rpt);
		//㷨ίб仯
		virtual void on_algo_order_status(AlgoOrder *order);
		//仯
		virtual void on_parameter(Parameter *param);
		//ʱ񴥷
		virtual void on_schedule(const char *data_rule, const char *time_rule);
		//زɺյЧ
		virtual void on_backtest_finished(Indicator *indicator);
		//ʵ˺״̬仯
		virtual void on_account_status(AccountStatus *account_status);
		//
		virtual void on_error(int error_code, const char *error_msg);
		//յֹͣź
		virtual void on_stop();
		//Ѿ
		virtual void on_market_data_connected();
		//Ѿ
		virtual void on_trade_data_connected();
		//ӶϿ
		virtual void on_market_data_disconnected();
		//ӶϿ
		virtual void on_trade_data_disconnected();

};




#endif
